#!/usr/bin/expectk

#------------------------------------------------------------------------------
# Hide main window until we're ready to display it:
#------------------------------------------------------------------------------

wm withdraw .

#------------------------------------------------------------------------------
# Configure:
#------------------------------------------------------------------------------
set ssh_binary "/usr/bin/ssh"
set sterminal_version 0.5

set default_host [lindex $argv 0]
set xsession     [lindex $argv 1]
set station_id   [lindex $argv 2]
set presession   [lindex $argv 3]
set atsession    [lindex $argv 4]
set postsession  [lindex $argv 5]
set bgcolor      [lindex $argv 6]
set bgimage      [lindex $argv 7]
set killcommand	 [lindex $argv 8]
set username     [lindex $argv 9]
set password     [lindex $argv 10]
set sshport      [lindex $argv 11]
set sshHostKeyAlias [lindex $argv 12]


set hostflag [string index $default_host 0]
if {"$hostflag" != "+"} {
    set host_locked 0
} else {
    set default_host [string trimleft $default_host "+"]
    set host_locked 1
}


#------------------------------------------------------------------------------
# Get display geometry:
#------------------------------------------------------------------------------
set width 480
set height 480
set swidth 256
set sheight 256
catch { exec ./getgeom.sh x} dwidth
catch { exec ./getgeom.sh y} dheight
set xcenter [expr $dwidth / 2]
set ycenter [expr $dheight / 2]
set xpos [expr $xcenter - [expr $width/2]]
set ypos [expr $ycenter - [expr $height/2]]
set sxpos [expr $xcenter - [expr $swidth/2]]
set sypos [expr $ycenter - [expr $sheight/2]]

#------------------------------------------------------------------------------
# Display splash screen:
#------------------------------------------------------------------------------
#toplevel .splash
#wm overrideredirect .splash 1
#image create photo splash
#splash configure -file sterminal-splash.gif
#canvas .splash.image -width 256 -height 256
#.splash.image create image 0 0 -image splash -anchor nw
#.splash.image create text 128 128 -text "S-Terminal version $sterminal_version" -anchor center
#pack configure .splash.image
#wm geometry .splash +$sxpos+$sypos

#------------------------------------------------------------------------------
# Fix bugs in Knoppix's debian distribution:
#------------------------------------------------------------------------------
#if [file exists /usr/lib/libtix.so.4.1] {
#    package ifneeded Tix 4.1 {load /usr/lib/libtix.so.4.1 Tix}
#} elseif [file exists /usr/lib/libtix.so.0] {
#    package ifneeded Tix 4.1 {load /usr/lib/libtix.so.0 Tix}
#} else {
#    puts "No Tix library available"
#    exit
#}
#package require Tix

#if [file exists /usr/lib/libexpect5.37.so.1] {
#    package ifneeded Expect 5.37 {load /usr/lib/libexpect5.37.so.1 Expect}
#}
#package require Expect
# Uncomment for debugging expect:
#exp_internal 1


######################################
# TakeHerDown - takes the machine down
# this is called when the "Shutdown"
# button is pushed on the main login
# screen.  The actual command
# is the "killcommand" in sterminal.conf
######################################
proc TakeHerDown { } {
  
  global killcommand

  # now we are back on our own machine
  # send the killcommand
  # first split the stupid thing
  set data [split $killcommand]
  exec [lindex $data 0] [lindex $data 1] [lindex $data 2] [lindex $data 3] [lindex $data 4]
  exit
  
}

######################################
# KillSterminal - takes the terminal down
# this is called when the "Kill Session"
# button is pushed (the annoying button
# which stays up the whole time)
######################################
proc KillSterminal { } {
  
  global ssh_slave
  global postsession

    # Execute Post-session commands, if any:
    catch {system $postsession} output

  # first kill ssh session
  exec kill -INT [exp_pid -i $ssh_slave]

  exit
  
}


#######################################################
# WaitForKill - displays "Kill Terminal" Button and
# waits for the button to be pushed, then shuts down
# computer (calls TakeHerDown)
#######################################################
proc WaitForKill { } {

    global xcenter
    global ycenter

    set width 100
    set height 30

    toplevel .waitforkill -relief ridge -borderwidth 3 -width $width -height $height
    button .waitforkill.ok -text "Kill Session" -command KillSterminal -default active 
    button .waitforkill.kill -text "Shutdown Machine" -command TakeHerDown -default active 
    pack configure .waitforkill.ok
    pack configure .waitforkill.kill
    bind .waitforkill <Key-Return> ".waitforkill.ok invoke"
    wm geometry .waitforkill +0+0

    # wait forever
    set waitvar 0
    while {$waitvar<1} {
      sleep 60
    }  

   # wait forever
#    set timeout -1
#    expect {
#	-re "(>|%|#|\\$) $" { exp_continue }
#    }

}

#------------------------------------------------------------------------------
# "BadLogin" message display procedure:
#------------------------------------------------------------------------------
proc BadLogin {} {

    global xcenter
    global ycenter

    set width 256
    set height 128
    set xpos [expr $xcenter - [expr $width/2]]
    set ypos [expr $ycenter - [expr $height/2]]

    toplevel .abort -relief ridge -borderwidth 3 -width $width -height $height
    label .abort.title -text "Error" -foreground white -background black
    label .abort.label -text "ssh Login failed. Try ssh manually?"
    button .abort.ok -text "OK" -command exit -default active 
    pack configure .abort.title -side top -expand yes -fill x
    pack configure .abort.label
    pack configure .abort.ok
    bind .abort <Key-Return> ".abort.ok invoke"
    wm geometry .abort +$xpos+$ypos
}

#------------------------------------------------------------------------------
# "About" message display procedure:
#------------------------------------------------------------------------------
proc About {} {
    global xcenter
    global ycenter
    global sterminal_version

    set width 480
    set height 256
    set xpos [expr $xcenter - [expr $width/2]]
    set ypos [expr $ycenter - [expr $height/2]]

    toplevel .about -relief ridge -borderwidth 3 -width $width -height $height
    label .about.title -text "About" -foreground white -background black
    label .about.label -justify center -text "S-Terminal $sterminal_version\nCopyright 2003, Bryan Wright <bkw1a@virginia.edu>"
    label .about.text -justify left -text "\n\nS-terminal is a secure X terminal, using an ssh-encrypted tunnel to \ntransport X traffic between client and server."
    button .about.ok -text "OK" -command "destroy .about" -default active 
    pack configure .about.title -side top -expand yes -fill x
    pack configure .about.label
    pack configure .about.text
    pack configure .about.ok
    bind .about <Key-Return> ".about.ok invoke"
    wm geometry .about +$xpos+$ypos
}

#------------------------------------------------------------------------------
# Procedure for connecting to host and running session command:
#------------------------------------------------------------------------------
proc ConnectHost {} {
    global username
    global password
    global sshport
    global hostname
    global session_cmd
    global ssh_binary
    global ssh_slave
    global xsession
    global station_id
    global presession
    global postsession
    global atsession
    global sshHostKeyAlias

    destroy .login.entry

    # Execute Pre-session commands, if any:
    catch {system $presession} output

    # last minute set this variable
    if {"$sshHostKeyAlias" == ""} {
      set sshHostKeyAlias $hostname
    }

    # Spawn off process to start session via ssh:
# NOTE THAT THE -console -noecho ARGUMENTS CAUSED CERTAIN VERSIONS
# OF EXPECT TO DIE, SO I TOOK THEM OUT
#    if [catch {spawn -console -noecho $ssh_binary -X $username@$hostname $xsession} output] {
#	BadLogin
#    }
    if [catch {spawn $ssh_binary -p $sshport -X -o HostKeyAlias=$sshHostKeyAlias $username@$hostname $xsession} output] {
	BadLogin
    }
    set ssh_slave $spawn_id

    set timeout 5

    # Deal with password dialog:
    expect {
	"assword:" {exp_send "$password\n"}
	timeout BadLogin
        "want to continue connecting" {
	    exp_send "yes\n"
	    expect {
		"assword:" {exp_send "$password\n"}
		timeout BadLogin
	    }
	}
    }
    
    destroy .login
    wm withdraw .

    # check if the password was bad
    expect {
      "assword:" BadLogin
    }

    # Execute Post-login commands, if any:
    catch {system $atsession} output

    # wait for a signal to shutdown sterminal (not the machine)
    WaitForKill

    # Execute Post-session commands, if any:
    catch {system $postsession} output

    exit
}

#------------------------------------------------------------------------------
# Create main (login) window:
#------------------------------------------------------------------------------
wm geometry . +$xpos+$ypos

#system xsetroot -solid $bgcolor

image create photo background
background configure -file $bgimage

canvas .login -width 480 -height 480 -background $bgcolor
.login create image 0 0 -image background -anchor nw

frame .login.entry -relief ridge -borderwidth 3

set hostname $default_host

frame .login.entry.banner -background black
label .login.entry.banner.label -background black -foreground white -text "S-Terminal version $sterminal_version" -justify center
button .login.entry.banner.about -text "?" -background black -foreground white -command About -relief flat
button .login.entry.banner.shutdown -text "Shutdown" -background black -foreground white -command TakeHerDown -relief flat
pack configure .login.entry.banner.about -side right
pack configure .login.entry.banner.shutdown -side left
pack configure .login.entry.banner.label -expand yes -fill x
pack configure .login.entry.banner -side top -expand yes -fill x -anchor center

frame .login.entry.name
label .login.entry.name.label -text "User Name: " -foreground blue 
entry .login.entry.name.entry -relief sunken -background white -textvariable username
pack configure .login.entry.name.entry -side right
pack configure .login.entry.name.label -anchor e
pack configure .login.entry.name -expand yes -fill x

frame .login.entry.password
label .login.entry.password.label -text "Password: " -foreground blue 
entry .login.entry.password.entry -show * -relief sunken -background white -textvariable password
pack configure .login.entry.password.entry -side right
pack configure .login.entry.password.label -anchor e
pack configure .login.entry.password -expand yes -fill x

frame .login.entry.hostname
if $host_locked {
    label .login.entry.hostname.label -text "Host Name: "
    label .login.entry.hostname.entry -text "$hostname"
    pack configure .login.entry.hostname.entry -side right
    pack configure .login.entry.hostname.label -anchor e
} else {
    label .login.entry.hostname.label -text "Host Name: " -foreground blue 
    entry .login.entry.hostname.entry -relief sunken -background white -textvariable hostname
    pack configure .login.entry.hostname.entry -side right
    pack configure .login.entry.hostname.label -anchor e
}
pack configure .login.entry.hostname -expand yes -fill x

frame .login.entry.xsession
label .login.entry.xsession.label -text "Command: " -foreground blue 
entry .login.entry.xsession.entry -relief sunken -background white -textvariable xsession
pack configure .login.entry.xsession.entry -side right
pack configure .login.entry.xsession.label -anchor e
pack configure .login.entry.xsession -expand yes -fill x

#frame .login.entry.id
#label .login.entry.id.label -foreground black -text "station id: $station_id" 
#pack configure .login.entry.id.label -expand yes -fill x -anchor center
#pack configure .login.entry.id -expand yes -fill x -anchor center

frame .login.entry.buttons
button .login.entry.buttons.ok -text "OK" -command ConnectHost -default active 
button .login.entry.buttons.cancel -text "Cancel" -command exit -default normal 
pack configure .login.entry.buttons.cancel -side left
pack configure .login.entry.buttons.ok -anchor e
pack configure .login.entry.buttons -fill x -expand yes

bind . <Key-Return> ".login.entry.buttons.ok invoke"

pack configure .login.entry

.login create window 240 240 -window .login.entry -anchor center

. configure -width 480 -height 480 -borderwidth 0
place .login -in . -x -1 -y -1

#------------------------------------------------------------------------------
# Display splash for at least 1 second:
#------------------------------------------------------------------------------
#sleep 1
#destroy .splash

#------------------------------------------------------------------------------
# Display main window:
#------------------------------------------------------------------------------
wm deiconify .
focus .login.entry.name.entry

